/*****************************************************************************
 * Copyright by The HDF Group.                                               *
 * Copyright by the Board of Trustees of the University of Illinois.         *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of the HDF Java Products distribution.                  *
 * The full copyright notice, including terms governing use, modification,   *
 * and redistribution, is contained in the file, COPYING.                    *
 * COPYING can be found at the root of the source code distribution tree.    *
 * If you do not have access to this file, you may request a copy from       *
 * help@hdfgroup.org.                                                        *
 ****************************************************************************/

/*
 *  For details of the HDF libraries, see the HDF Documentation at:
 *    https://support.hdfgroup.org/documentation/
 *
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "hdf5.h"
#include <stdlib.h>
#include "h5jni.h"
#include "h5plImp.h"

extern JavaVM *jvm;
extern jobject visit_callback;

/*
 * Class:     hdf_hdf5lib_H5
 * Method:    H5PLset_loading_state
 * Signature: (I)V
 */
JNIEXPORT void JNICALL
Java_hdf_hdf5lib_H5_H5PLset_1loading_1state
    (JNIEnv *env, jclass clss, jint plugin_flags)
{
#if (1)
    if (H5PLset_loading_state((unsigned int)plugin_flags) < 0) {
        h5libraryError(env);
    }
#else
    h5unimplemented(env, "H5PLset_loading_state: only available > 1.8.14");
#endif
} /* end Java_hdf_hdf5lib_H5_H5PLset_1loading_1state */

/*
 * Class:     hdf_hdf5lib_H5
 * Method:    H5PLget_loading_state
 * Signature: (V)I
 */
JNIEXPORT jint JNICALL
Java_hdf_hdf5lib_H5_H5PLget_1loading_1state
    (JNIEnv *env, jclass clss)
{
    unsigned int plugin_type = 0;
#if (1)
    if (H5PLget_loading_state(&plugin_type) < 0) {
        h5libraryError(env);
    }
#else
    h5unimplemented(env, "H5PLget_loading_state: only available > 1.8.14");
#endif
    return (jint)plugin_type;
} /* end Java_hdf_hdf5lib_H5_H5PLget_1loading_1state */

#ifdef __cplusplus
} /* end extern "C" */
#endif /* __cplusplus */
